@@ -1,54 +1,94 @@
 import contextlib
+import errno
 import os
 import sys
 import tempfile
 
-__version__ = '0.1.2'
+__version__ = '0.1.3'
+
+
+PY2 = sys.version_info[0] == 2
+
+if PY2:
+    class FileExistsError(OSError):
+        errno = errno.EEXIST
+else:
+    # For some reason we have to redefine this, or users can't import it.
+    FileExistsError = FileExistsError
+
 
 if sys.platform != 'win32':
-    def replace_atomic(src, dst):
+    @contextlib.contextmanager
+    def handle_errors():
+        try:
+            yield
+        except FileExistsError:
+            raise
+        except OSError as e:
+            if e.errno == errno.EEXIST:
+                raise FileExistsError(str(e))
+            else:
+                raise
+
+    def _replace_atomic(src, dst):
         os.rename(src, dst)
 
-    def move_atomic(src, dst):
+    def _move_atomic(src, dst):
         os.link(src, dst)
         os.unlink(src)
 else:
     import win32api
-    import win32con
+    import win32file
+    import pywintypes
 
-    _windows_default_flags = win32con.WRITE_THROUGH
+    _windows_default_flags = win32file.MOVEFILE_WRITE_THROUGH
 
-    def replace_atomic(src, dst):
+    @contextlib.contextmanager
+    def handle_errors():
+        try:
+            yield
+        except pywintypes.error as e:
+            if e.winerror == 183:
+                raise FileExistsError(str(e))
+            else:
+                raise
+
+    def _replace_atomic(src, dst):
         win32api.MoveFileEx(
             src, dst,
-            win32con.MOVEFILE_REPLACE_EXISTING | _windows_default_flags
+            win32file.MOVEFILE_REPLACE_EXISTING | _windows_default_flags
         )
 
-    def move_atomic(src, dst):
+    def _move_atomic(src, dst):
         win32api.MoveFileEx(
             src, dst,
             _windows_default_flags
         )
 
 
-replace_atomic.__doc__ = \
+def replace_atomic(src, dst):
     '''
     Move ``src`` to ``dst``. If ``dst`` exists, it will be silently
     overwritten.
 
     Both paths must reside on the same filesystem for the operation to be
     atomic.
-    ''' + (replace_atomic.__doc__ or '')
+    '''
+    with handle_errors():
+        return _replace_atomic(src, dst)
 
 
-move_atomic.__doc__ = \
+def move_atomic(src, dst):
     '''
     Move ``src`` to ``dst``. There might a timewindow where both filesystem
-    entries exist. If ``dst`` already exists, an error will be raised.
+    entries exist. If ``dst`` already exists, :py:exc:`FileExistsError` will be
+    raised.
 
     Both paths must reside on the same filesystem for the operation to be
     atomic.
-    ''' + (move_atomic.__doc__ or '')
+    '''
+    with handle_errors():
+        return _move_atomic(src, dst)
 
 
 class AtomicWriter(object):
@@ -58,18 +98,6 @@ class AtomicWriter(object):
         with AtomicWriter(path).open() as f:
             f.write(...)
 
-
-    It uses a temporary file in the same directory as the given path. This
-    ensures that the temporary file resides on the same filesystem.
-
-    The temporary file will then be atomically moved to the target location: On
-    POSIX, it will use ``rename`` if files should be overwritten, otherwise a
-    combination of ``link`` and ``unlink``. On Windows, it uses ``MoveFileEx``
-    (see MSDN_) with the appropriate flags.
-
-    .. _MSDN::
-        https://msdn.microsoft.com/en-us/library/windows/desktop/aa365240%28v=vs.85%29.aspx
-
     :param path: The destination filepath. May or may not exist.
     :param overwrite: If set to false, an error is raised if ``path`` exists.
         Either way, the operation is atomic.
@@ -90,11 +118,13 @@ class AtomicWriter(object):
         try:
             with get_fileobject() as f:
                 yield f
-        except:
-            self.rollback()
-            raise
-        else:
             self.commit()
+        except:
+            try:
+                self.rollback()
+            except Exception:
+                pass
+            raise
 
     def get_fileobject(self, mode):
         '''Return the temporary path to use.'''
@@ -106,10 +136,9 @@ class AtomicWriter(object):
     def commit(self):
         '''Move the temporary file to the target location.'''
         if self._overwrite:
-            os.rename(self._tmppath, self._path)  # atomic
+            replace_atomic(self._tmppath, self._path)  # atomic
         else:
-            os.link(self._tmppath, self._path)  # atomic, fails if file exists
-            os.unlink(self._tmppath)
+            move_atomic(self._tmppath, self._path)
 
     def rollback(self):
         '''Clean up all temporary resources.'''
@@ -119,8 +148,7 @@ class AtomicWriter(object):
 def atomic_write(path, mode='w', overwrite=False, writer_cls=AtomicWriter,
                  **open_kwargs):
     '''
-    Simple atomic writes. This wraps :py:class:`AtomicWriterBase` and its
-    subclasses::
+    Simple atomic writes. This wraps :py:class:`AtomicWriter`::
 
         with atomic_write(path) as f:
             f.write(...)
